package com.usky.system.mapper;


import com.usky.common.mybatis.core.CrudMapper;
import com.usky.system.domain.SysUser;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;


/**
 * 用户表 数据层
 *
 * @author yq
 */
@Repository
public interface SysUserMapper extends CrudMapper<SysUser> {
    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUserList(SysUser sysUser);

    /**
     * 根据条件分页查询未已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUnallocatedList(SysUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public SysUser selectUserByUserName(String userName);

    public SysUser selectUserByUserNameOne(@Param("userName") String userName, @Param("tenantId") Integer tenantId);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public SysUser selectUserById(Long userId);

    public SysUser selectUserByIdOne(Long userId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(SysUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar   头像地址
     * @return 结果
     */
    public int updateUserAvatar(@Param("userName") String userName, @Param("avatar") String avatar);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(@Param("userName") String userName, @Param("password") String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param ids 需要删除的用户ID
     * @return 结果
     */
    public int deleteUserByIds(@Param("ids") Long[] ids);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    public int checkUserNameUnique(@Param("userName") String userName, @Param("tenantId") Integer tenantId);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public SysUser checkPhoneUnique(String phonenumber);

    /**
     * 校验手机号码是否唯一
     *
     * @param phone 手机号
     * @param tenantId 租户ID
     * @return 结果
     */
    public int checkPhoneUnique1(@Param("phone") String phone, @Param("tenantId") Integer tenantId);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public SysUser checkEmailUnique(String email);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @param tenantId 租户ID
     * @return 结果
     */
    public int checkEmailUnique1(@Param("email") String email, @Param("tenantId") Integer tenantId);

    public int resetUserPwdOne(@Param("userId") Long userId, @Param("password") String password);

    public List<SysUser> getUserData(@Param("tenantId") Integer tenantId,
                                     @Param("current") Integer current,
                                     @Param("size") Integer size,
                                     @Param("userId") long userId,
                                     @Param("nickName") String nickName);

    public List<SysUser> userList(@Param("tenantId") Integer tenantId,
                                  @Param("userName") String userName);

    SysUser selectUserData(@Param("userName") String userName,
                           @Param("tenantId") Integer tenantId);

    SysUser selectUserDataOne(@Param("tenantId") Integer tenantId,
                              @Param("phone") String phone);

    public List<SysUser> userAllList();

    public List<SysUser> tenantIdUserList(@Param("tenantId") Integer tenantId);

    /**
     * 查询mce_mbuser消息中心_移动端注册表对应的userId记录
     */
    public int selectMbUserCount(@Param("userId") Long userId);

    public void insertMbUser(@Param("userId") Long userId,@Param("cids") String cids,@Param("createBy") String createBy);

    public void updateMbUser(@Param("phone") String phone,@Param("openId") String openId,@Param("userId") Long userId,@Param("cids") String cids,@Param("updateBy") String updateBy);

    SysUser selectUserTenantData(@Param("userName") String userName,
                                 @Param("tenantId") Integer tenantId);

    SysUser selectUserTenantDataOne(@Param("tenantId") Integer tenantId,
                                    @Param("phone") String phone);

    SysUser selectUserTenantData1(@Param("userName") String userName,
                                 @Param("tenantId") Integer tenantId);

    SysUser selectUserTenantDataOne1(@Param("tenantId") Integer tenantId,
                                    @Param("phone") String phone);
}
